#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2008 MWneo Corporation
# Copyright (C) 2018 Omega Tech Enterprises Ltd.
# (All rights transferred from MWneo Corporation to Omega Tech Enterprises Ltd.)
#

# Runs a "main" python script, adding one or more zip packages to the path
# Usage: python runpackage.py package.pypkg [package2.pypkg] [...] [--args <program args>]
#

import argparse
import logging
import logging.config
import logging.handlers

import os
import platform
import distro
import sys
import traceback
import zipfile
import shutil


class RunPackage():
    def __init__(self):
        
        self.VERSION = "1.2.0"
        self.MWPLATFORM = self.get_platform()
        self.ctl_file = None

        # environment variables
        self.HVDATADIR = os.environ.get('HVDATADIR')
        self.BUNDLEDIR = os.environ.get('BUNDLEDIR')
        self.LOG_PATH = os.environ.get('LOG_PATH')

        self.comp_name = self.BUNDLEDIR.split("/")[-2]
        runpackage_dir = os.path.join('RunPackage')
        os.makedirs(runpackage_dir, exist_ok=True)
        self.config_logger(self.comp_name)
        self.logger = logging.getLogger("RunPackage_{}".format(self.comp_name))
        
        self.parse_args()
        self.add_paths()
        self.unpack()

    def get_platform(self):
        # determine platform
        system = (platform.system() or '').lower()
        distribution = '-{0}'.format(distro.id().lower()) if distro.id() else ''
        ret = '{0}{1}-{2}'.format(system, distribution, platform.machine())
        if ret.lower().startswith('windows'):
            ret = 'windows-x86'
        return ret

    def parse_args(self):
        description = """Helper script to run MW:APP python components"""
        parser = argparse.ArgumentParser(description=description, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
        parser.add_argument('pkgs', nargs='+', help='packages or paths to add at runtime', metavar='PYPKG')
        parser.add_argument('--args', nargs='*', help='arguments to be passed to main()')
        parser.add_argument('--unpack-dir', help='unpack directory', default='{BUNDLEDIR}', metavar='DIR')
        parser.add_argument('--unpack', nargs='*', help='extract a directory from pypkg (e.g. mypkg.pypkg/lib)', metavar='PYPKG/dir')
        parser.add_argument('--version', action='version', version='%(prog)s v{0}'.format(self.VERSION))
        self.args = parser.parse_args()

    def add_paths(self):
        # add packages and other directories to system path
        for i, arg in enumerate(self.args.pkgs):
            if self.BUNDLEDIR and "{BUNDLEDIR}" in arg:
                self.args.pkgs[i] = arg.replace("{BUNDLEDIR}", self.BUNDLEDIR)

        if self.LOG_PATH and self.BUNDLEDIR and "{BUNDLEDIR}" in self.LOG_PATH:
            os.environ["LOG_PATH"] = self.LOG_PATH.replace("{BUNDLEDIR}", self.BUNDLEDIR)

        sys.path.extend(self.args.pkgs or [])
        # extend system path to find platform-specific lib directories
        if self.HVDATADIR:
            sys.path.append(os.path.join(self.HVDATADIR, 'common', 'lib', self.MWPLATFORM))
        if self.BUNDLEDIR:
            sys.path.append(os.path.join(self.BUNDLEDIR, 'lib', self.MWPLATFORM))
            sys.path.append(os.path.join(self.BUNDLEDIR, 'python', 'lib', self.MWPLATFORM))

    def read_file(self, path):
        if os.path.exists(path):
            with open(path) as f:
                return f.read()
        return None

    def write_file(self, content):
        if self.ctl_file is None:
            return

        with open(self.ctl_file, "w+") as f:
            f.write(content)

    def reset_unpack(self):
        if self.ctl_file is None:
            return

        if not os.path.exists(self.ctl_file):
            return
        os.remove(self.ctl_file)

    def unpack(self):
        # if --unpack-dir was not passed, expand the default value. If it was passed on the loader,
        # it will be already expanded by hypervisor
        if self.args.unpack_dir == '{BUNDLEDIR}':
            self.args.unpack_dir = os.path.join(self.BUNDLEDIR, '.')

        # check if unpack is necessary for this component
        if self.args.unpack:
            # make sure that output directory exists
            try:
                os.makedirs(self.args.unpack_dir)
            except:
                pass

            sl_flag = 120000 << 16  # symlink file type

            stamp = None
            for path in (self.args.unpack or []):
                path_splitted = path.split('.pypkg/')
                if len(path_splitted) != 2:
                    self.logger.exception("Invalid unpack parameter [{}], expected 'mypackage.pypkg/dir'".format(path))
                    continue

                pypkg, extract_dir = ('{0}.pypkg'.format(path_splitted[0]), path_splitted[1])
                if not os.path.exists(pypkg):
                    self.logger.exception("PYPKG file not found [{}]".format(pypkg))
                    continue

                stamp = "{0}|{1}".format(os.path.getmtime(pypkg), os.path.getsize(pypkg))
                self.ctl_file = os.path.join(self.args.unpack_dir, '.{0}.ctl'.format(os.path.basename(pypkg)))
                full_extract_dir = os.path.join(self.args.unpack_dir, extract_dir)
                if self.read_file(self.ctl_file) == stamp and os.path.exists(full_extract_dir):
                    # nothing to update
                    continue

                try:
                    archive = zipfile.ZipFile(pypkg)
                    symlinks = []
                    members = [member for member in archive.namelist() if member.startswith(extract_dir)]
                    for member in members:
                        zipinfo = archive.getinfo(member)
                        if (zipinfo.external_attr & sl_flag) == sl_flag:
                            symlink_data = archive.read(member)
                            symlinks.append((symlink_data, os.path.join(self.args.unpack_dir, member)))
                        else:
                            archive.extract(member, path=self.args.unpack_dir)
                    # create symlinks
                    for symlink, dst in symlinks:
                        try:
                            os.symlink(symlink, dst)
                        except:
                            pass
                except:
                    self.logger.exception("Exception extracting {}".format(pypkg))
                    break
            else:
                if stamp is not None:
                    self.write_file(stamp)

    def run(self):
        main = None
        try:
            import main
            # Check if the "main" function accepts arguments
            if main.main.__code__.co_argcount == 1:
                main.main(self.args.args or [])
            else:
                main.main()
        except:
            message = "Error starting module"
            if main:
                message = "Error on module: {}".format(main)
            self.logger.exception(message)
            traceback.print_exc(file=sys.stdout)
            self.reset_unpack()
            raise

    @staticmethod
    def config_logger(comp_name):
        filename = os.path.join('RunPackage', "{}.log".format(comp_name))

        format_str = "%(asctime)-6s: %(name)s - %(levelname)s - %(thread)d - %(threadName)s - %(message)s"
        formatter = logging.Formatter(format_str)

        comp_logger = logging.getLogger("RunPackage_{}".format(comp_name))
        comp_logger.setLevel(logging.DEBUG)
        comp_logger.propagate = False

        rotating_file_logger = logging.handlers.RotatingFileHandler(filename, 'a', 5242880, 1)
        rotating_file_logger.setLevel(logging.DEBUG)
        rotating_file_logger.setFormatter(formatter)

        comp_logger.addHandler(rotating_file_logger)


if __name__ == "__main__":
    runpackage = RunPackage()
    runpackage.run()



#-*- coding: ISO-8859-1 -*-
# pysqlite2/test/transactions.py: tests transactions
#
# Copyright (C) 2005-2007 Gerhard Häring <gh@ghaering.de>
#
# This file is part of pysqlite.
#
# This software is provided 'as-is', without any express or implied
# warranty.  In no event will the authors be held liable for any damages
# arising from the use of this software.
#
# Permission is granted to anyone to use this software for any purpose,
# including commercial applications, and to alter it and redistribute it
# freely, subject to the following restrictions:
#
# 1. The origin of this software must not be misrepresented; you must not
#    claim that you wrote the original software. If you use this software
#    in a product, an acknowledgment in the product documentation would be
#    appreciated but is not required.
# 2. Altered source versions must be plainly marked as such, and must not be
#    misrepresented as being the original software.
# 3. This notice may not be removed or altered from any source distribution.

import sys
import os, unittest
import sqlite3 as sqlite

def get_db_path():
    return "sqlite_testdb"

class TransactionTests(unittest.TestCase):
    def setUp(self):
        try:
            os.remove(get_db_path())
        except OSError:
            pass

        self.con1 = sqlite.connect(get_db_path(), timeout=0.1)
        self.cur1 = self.con1.cursor()

        self.con2 = sqlite.connect(get_db_path(), timeout=0.1)
        self.cur2 = self.con2.cursor()

    def tearDown(self):
        self.cur1.close()
        self.con1.close()

        self.cur2.close()
        self.con2.close()

        try:
            os.unlink(get_db_path())
        except OSError:
            pass

    def CheckDMLdoesAutoCommitBefore(self):
        self.cur1.execute("create table test(i)")
        self.cur1.execute("insert into test(i) values (5)")
        self.cur1.execute("create table test2(j)")
        self.cur2.execute("select i from test")
        res = self.cur2.fetchall()
        self.assertEqual(len(res), 1)

    def CheckInsertStartsTransaction(self):
        self.cur1.execute("create table test(i)")
        self.cur1.execute("insert into test(i) values (5)")
        self.cur2.execute("select i from test")
        res = self.cur2.fetchall()
        self.assertEqual(len(res), 0)

    def CheckUpdateStartsTransaction(self):
        self.cur1.execute("create table test(i)")
        self.cur1.execute("insert into test(i) values (5)")
        self.con1.commit()
        self.cur1.execute("update test set i=6")
        self.cur2.execute("select i from test")
        res = self.cur2.fetchone()[0]
        self.assertEqual(res, 5)

    def CheckDeleteStartsTransaction(self):
        self.cur1.execute("create table test(i)")
        self.cur1.execute("insert into test(i) values (5)")
        self.con1.commit()
        self.cur1.execute("delete from test")
        self.cur2.execute("select i from test")
        res = self.cur2.fetchall()
        self.assertEqual(len(res), 1)

    def CheckReplaceStartsTransaction(self):
        self.cur1.execute("create table test(i)")
        self.cur1.execute("insert into test(i) values (5)")
        self.con1.commit()
        self.cur1.execute("replace into test(i) values (6)")
        self.cur2.execute("select i from test")
        res = self.cur2.fetchall()
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0][0], 5)

    def CheckToggleAutoCommit(self):
        self.cur1.execute("create table test(i)")
        self.cur1.execute("insert into test(i) values (5)")
        self.con1.isolation_level = None
        self.assertEqual(self.con1.isolation_level, None)
        self.cur2.execute("select i from test")
        res = self.cur2.fetchall()
        self.assertEqual(len(res), 1)

        self.con1.isolation_level = "DEFERRED"
        self.assertEqual(self.con1.isolation_level , "DEFERRED")
        self.cur1.execute("insert into test(i) values (5)")
        self.cur2.execute("select i from test")
        res = self.cur2.fetchall()
        self.assertEqual(len(res), 1)

    def CheckRaiseTimeout(self):
        if sqlite.sqlite_version_info < (3, 2, 2):
            # This will fail (hang) on earlier versions of sqlite.
            # Determine exact version it was fixed. 3.2.1 hangs.
            return
        self.cur1.execute("create table test(i)")
        self.cur1.execute("insert into test(i) values (5)")
        try:
            self.cur2.execute("insert into test(i) values (5)")
            self.fail("should have raised an OperationalError")
        except sqlite.OperationalError:
            pass
        except:
            self.fail("should have raised an OperationalError")

    def CheckLocking(self):
        """
        This tests the improved concurrency with pysqlite 2.3.4. You needed
        to roll back con2 before you could commit con1.
        """
        if sqlite.sqlite_version_info < (3, 2, 2):
            # This will fail (hang) on earlier versions of sqlite.
            # Determine exact version it was fixed. 3.2.1 hangs.
            return
        self.cur1.execute("create table test(i)")
        self.cur1.execute("insert into test(i) values (5)")
        try:
            self.cur2.execute("insert into test(i) values (5)")
            self.fail("should have raised an OperationalError")
        except sqlite.OperationalError:
            pass
        except:
            self.fail("should have raised an OperationalError")
        # NO self.con2.rollback() HERE!!!
        self.con1.commit()

    def CheckRollbackCursorConsistency(self):
        """
        Checks if cursors on the connection are set into a "reset" state
        when a rollback is done on the connection.
        """
        con = sqlite.connect(":memory:")
        cur = con.cursor()
        cur.execute("create table test(x)")
        cur.execute("insert into test(x) values (5)")
        cur.execute("select 1 union select 2 union select 3")

        con.rollback()
        try:
            cur.fetchall()
            self.fail("InterfaceError should have been raised")
        except sqlite.InterfaceError, e:
            pass
        except:
            self.fail("InterfaceError should have been raised")

class SpecialCommandTests(unittest.TestCase):
    def setUp(self):
        self.con = sqlite.connect(":memory:")
        self.cur = self.con.cursor()

    def CheckVacuum(self):
        self.cur.execute("create table test(i)")
        self.cur.execute("insert into test(i) values (5)")
        self.cur.execute("vacuum")

    def CheckDropTable(self):
        self.cur.execute("create table test(i)")
        self.cur.execute("insert into test(i) values (5)")
        self.cur.execute("drop table test")

    def CheckPragma(self):
        self.cur.execute("create table test(i)")
        self.cur.execute("insert into test(i) values (5)")
        self.cur.execute("pragma count_changes=1")

    def tearDown(self):
        self.cur.close()
        self.con.close()

def suite():
    default_suite = unittest.makeSuite(TransactionTests, "Check")
    special_command_suite = unittest.makeSuite(SpecialCommandTests, "Check")
    return unittest.TestSuite((default_suite, special_command_suite))

def test():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    test()
